/* 
 * File:   TTSEventQueue.h
 * Author: matt
 *
 * Created on July 14, 2011, 3:48 AM
 */

#ifndef TTSEVENTQUEUE_H
#define	TTSEVENTQUEUE_H

#include "robokind/messaging/AQSender.h"
#include "robokind/speech/protocol/SpeechEventRecord.h"

namespace robokind{
  namespace acapela{
      
    class TTSEventQueue : public EventCallback{
        private:
            AQSender* mySender;
            
            SpeechEventRecord mapEvent(unsigned int eventId, PNSC_EVENT_DATA eventData){
                SpeechEventRecord event;
                switch (eventId) {
                    //These are all that are currently supported
                    case NSC_EVID_TEXT_STARTED:{
                        NSC_EVENT_DATA_TextStarted *pEVTS = (NSC_EVENT_DATA_TextStarted *) eventData;
                        event.eventType = "SPEECH_START";
                        event.streamNumber = (int64_t) pEVTS->pUserData;
                        break;
                    }case NSC_EVID_TEXT_DONE:{
                        NSC_EVENT_DATA_TextDone *pEVTD = (NSC_EVENT_DATA_TextDone *) eventData;
                        event.eventType = "SPEECH_END";
                        event.streamNumber = (int64_t) pEVTD->pUserData;
                        break;
                    }case NSC_EVID_WORD_SYNCH:{
                        NSC_EVENT_DATA_WordSynch *pEVWS = (NSC_EVENT_DATA_WordSynch *) eventData;
                        event.eventType = "WORD_BOUNDARY";
                        event.streamNumber = (int64_t) pEVWS->pUserData;
                        //event.bytePosition = pEVWS->uiByteCount;
                        event.textPosition = pEVWS->uiWordPos;
                        break;
                    }case NSC_EVID_BOOKMARK:{
                        NSC_EVENT_DATA_Bookmark *pEVBM = (NSC_EVENT_DATA_Bookmark *) eventData;
                        event.eventType = "BOOKMARK";
                        event.streamNumber = (int64_t) pEVBM->pUserData;
                        //event.bytePosition = pEVBM->uiByteCount;
                        char* str = new char[2];
                        str[0] = (char)pEVBM->uiVal;
                        str[0] = '\0';
                        event.stringData = str;
                        break;
                    }   
                }
                return event;
            }
            
        public:
            TTSEventQueue(AQSender* sender){
                mySender = sender;
            }
            
            int callback(unsigned int eventId, unsigned int len, PNSC_EVENT_DATA eventData,  void * user_data){
                if(mySender == NULL){
                    return 0;
                }
                SpeechEventRecord event = mapEvent(eventId, eventData);
                mySender->send<SpeechEventRecord>(&event);
                return 0;
            }
    };
    
}}

#endif	/* TTSEVENTQUEUE_H */

