/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef SPEECHEVENTLISTRECORD_H_2636285834__H_
#define SPEECHEVENTLISTRECORD_H_2636285834__H_


#include <boost/any.hpp>
#include <avro/Specific.hh>
#include <avro/Encoder.hh>
#include <avro/Decoder.hh>
#include "robokind/speech/protocol/SpeechEventRecord.h"

struct SpeechEventListRecord {
    std::string speechServiceId;
    int64_t timestampMillisecUTC;
    std::vector<SpeechEventRecord> speechEvents;
};

namespace avro {
template<> struct codec_traits<SpeechEventListRecord> {
    static void encode(Encoder& e, const SpeechEventListRecord& v) {
        avro::encode(e, v.speechServiceId);
        avro::encode(e, v.timestampMillisecUTC);
        avro::encode(e, v.speechEvents);
    }
    static void decode(Decoder& d, SpeechEventListRecord& v) {
        avro::decode(d, v.speechServiceId);
        avro::decode(d, v.timestampMillisecUTC);
        avro::decode(d, v.speechEvents);
    }
};

}
#endif
