/* 
 * File:   main.cpp
 * Author: matt
 *
 * Created on November 12, 2011, 3:59 PM
 */

#include <cstdlib>
#include "robokind/messaging/AQStream.h"
#include "robokind/messaging/AQSender.h"
#include "robokind/messaging/AQService.h"
#include "robokind/messaging/AQServiceProvider.h"
#include "robokind/vision/proc/facedetect/FaceDetectService.h"
#include "robokind/vision/proc/facedetect/protocol/FaceDetectConfig.h"

using namespace std;
using namespace qpid::messaging;
using namespace robokind;

const char* connectionStr = "localhost:5672";
const char* connectionOpt = "{username:admin,password:admin}";
const char* destVidQ = "vision.VideoTopic1; {create: always, node: {type: topic}}";
const char* destFD = "vision.FaceTopic1; {create: always, node: {type: topic}}";
const char* destFDCmd = "vision.FaceCmdQueue1; {create: always, node: {type: queue}}";
const char* destFDErr = "vision.FaceErrQueue1; {create: always, node: {type: queue}}";

FaceDetectConfig* getDefaultFaceDetectConfig(){
	FaceDetectConfig* conf = new FaceDetectConfig();
    conf->brokerAddress = connectionStr;
    conf->connectionOptions = connectionOpt;
    conf->receivingAddress = destVidQ;
    conf->resultsAddress = destFD;
    conf->haarFileLocation = "haarcascade_frontalface_default.xml";
    conf->scaleFactor = 1.5;
    conf->neighbors = 3;
    conf->haarFlags = 0;
    conf->minFaceWidth = 40;
    conf->minFaceHeight = 40;
    return conf;
}

template<typename T> void provider(AQService<T>* service, const char* destCmd, const char* destErr){
    AQStream* myStream;
    myStream = new AQStream();
    if(!myStream->connect())
        return;
    AQReceiver* cmdRec = myStream->createReceiver(destCmd);
    AQSender* errSnd = myStream->createSender(destErr);
    AQServiceProvider<T> s(service, cmdRec, errSnd);
    s.start();
}

int main(int argc, char** argv) {
    cout << "Starting Face Detect Service." << endl;
	FaceDetectService* service = new FaceDetectService();
	FaceDetectConfig* config;
	if(argc <= 1){
		cout << "No file specified, loading default config." << endl;
		config = getDefaultFaceDetectConfig();
	}else{
		config = loadFromFile<FaceDetectConfig>(argv[1]);
	}
	if(config != NULL){
		service->initialize(config);
	}
    provider<FaceDetectConfig>(service, destFDCmd, destFDErr);
    return 0;
}


