#include <iostream>
#include "robokind/messaging/AQStream.h"
#include "robokind/messaging/AQSender.h"
#include "robokind/messaging/AQService.h"
#include "robokind/messaging/AQServiceProvider.h"
#include "robokind/speech/SpeechEngine.h"
#include "robokind/speech/SpeechService.h"
#include "robokind/speech/mssapi/SAPIEngine.h"
#include "robokind/speech/protocol/SpeechConfigRecord.h"
#include "robokind/speech/protocol/SpeechRequestRecord.h"
#include "robokind/speech/protocol/SpeechEventListRecord.h"
#include "robokind/speech/protocol/SpeechEventRecord.h"

using namespace std;
using namespace qpid::messaging;
using namespace robokind;
using namespace robokind::speech;

const char* connectionStr = "localhost:5672";
const char* connectionOpt = "{username:admin,password:admin}";
const char* requestQueue = "speech.RequestQueue1; {create: always, node: {type: queue}}";
const char* eventQueue = "speech.EventQueue1; {create: always, node: {type: queue}}";
const char* cmdQueue = "speech.CommandQueue1; {create: always, node: {type: queue}}";
const char* errQueue = "speech.ErrorQueue1; {create: always, node: {type: queue}}";

SpeechConfigRecord* getDefaultSpeechConfig(){
	SpeechConfigRecord* conf = new SpeechConfigRecord();
    conf->brokerAddress = connectionStr;
    conf->connectOptions = connectionOpt;
    conf->requestQueue = requestQueue;
    conf->eventQueue = eventQueue;
    conf->voiceName = "nelly";
    conf->sampleRate = 160000.0;
    return conf;
}

template<typename T> void provider(AQService<T>* service, const char* destCmd, const char* destErr){
    AQStream* myStream;
    myStream = new AQStream();
    if(!myStream->connect())
        return;
    AQReceiver* cmdRec = myStream->createReceiver(destCmd);
    AQSender* errSnd = myStream->createSender(destErr);
    AQServiceProvider<T> s(service, cmdRec, errSnd);
    s.start();
}

int main(int argc, char **argv) {
    cout << "Starting Text-to-Speech Service." << endl;
	AQService<SpeechConfigRecord>* service = new SpeechService((SpeechEngine*)SAPIEngine::instance());
	SpeechConfigRecord* config;
	if(argc <= 1){
		cout << "No file specified, loading default config." << endl;
		config = getDefaultSpeechConfig();
	}else{
		config = loadFromFile<SpeechConfigRecord>(argv[1]);
	}
	if(config != NULL){
		service->initialize(config);
	}
	provider<SpeechConfigRecord>(service, cmdQueue, errQueue);
	return 0;
}

