#include "robokind/speech/SpeechService.h"
#include "robokind/speech/protocol/SpeechRequestRecord.h"

using namespace robokind::speech;


SpeechService::SpeechService(SpeechEngine* engine):mySpeechEngine(engine){}

void SpeechService::threadSafeWorker(){
	Message msg;
	try{
		msg = myRequestReceiver->fetchRaw(Duration(1000));
	}catch(const NoMessageAvailable &ex){
		return;
	}
	SpeechRequestRecord* req = unpackMessage<SpeechRequestRecord>(msg);
	if(req == NULL){
		cout << "Could not unpack message." << endl;
		myRequestReceiver->acknowledge(); 
		return;
	}
	cout << "Received Phrase: " << req->phrase << endl;
	mySpeechEngine->speak(req->phrase);
	myRequestReceiver->acknowledge();
}

bool SpeechService::_initialize(SpeechConfigRecord* conf){
    myStream = new AQStream(conf->brokerAddress.c_str(), conf->connectOptions.c_str());
    myStream->connect();
    myRequestReceiver = myStream->createReceiver(conf->requestQueue.c_str());
    myEventSender = myStream->createSender(conf->eventQueue.c_str());
    bool ret = mySpeechEngine->initialize(conf->voiceName, conf->sampleRate, myEventSender);
	mySpeechEngine->startEventListener();
	return ret;
}

void SpeechService::close(){
    mySpeechEngine->stop();
	myStream->disconnect();
}
void SpeechService::handleCommandMessage(Message &msg){}
