#include "robokind/vision/camera/WinCameraService.h"
#include <sys/time.h>
#include <windows.h>

using namespace robokind;

#define DEF_CAMERA_WIDTH 640
#define DEF_CAMERA_HEIGHT 480
#define DEF_CAMERA_CHANNELS 3

bool WinCameraService::_initialize(CameraConfig* conf){
    AQStream* stream = new AQStream(
            conf->brokerAddress.c_str(), conf->connectionOptions.c_str());
    stream->connect();
    myFrameLength = (int)conf->frameLength;
    myDevice = (int)conf->cameraNumber;
    string dest = conf->destination;
    myVideoSender = stream->createSender(dest.c_str());
    
    int c = conf->imageChannels;
    int w = conf->imageWidth;
    int h = conf->imageHeight;

	myVideo.setUseCallback(true);
	myVideo.setIdealFramerate(myDevice, myFrameLength);
	myVideo.setupDevice(myDevice, DEF_CAMERA_WIDTH, DEF_CAMERA_HEIGHT, VI_COMPOSITE);
    
    myConvertColorFlag = (c != DEF_CAMERA_CHANNELS);
    myConvertColorHeader = cvCreateImage(cvSize(DEF_CAMERA_WIDTH,DEF_CAMERA_HEIGHT),IPL_DEPTH_8U,c);
    
    myResizeFlag = (w != DEF_CAMERA_WIDTH || h != DEF_CAMERA_HEIGHT);
    myResizeHeader = cvCreateImage(cvSize(w,h),IPL_DEPTH_8U,c);
	myImageData = new unsigned char[myVideo.getSize(myDevice)];

	return true;
}

void WinCameraService::close(){

}

void WinCameraService::threadSafeWorker(){
    try{
		int start = static_cast<int>(GetTickCount());
        IplImage* img = getImage();
		if(img == NULL){
            boost::this_thread::sleep(boost::posix_time::milliseconds(3));
			return;
		}
        IplImage* temp = img;
        if(myConvertColorFlag){
            cvConvertImage(temp,myConvertColorHeader);
            temp = myConvertColorHeader;
        }
        if(myResizeFlag){
            cvResize(temp,myResizeHeader);
            temp = myResizeHeader;
        }
        ImageRecord *pimg = packImage(temp);
        myVideoSender->send<ImageRecord>(pimg, "application/image-record", true);
        
		
		int end = static_cast<int>(GetTickCount());
        long elapsed = end - start;
        long sleep = myFrameLength - elapsed;
        if(sleep > 0){
            boost::this_thread::sleep(boost::posix_time::milliseconds(sleep));
		}
        delete pimg;
    }catch(const cv::Exception &ex){
        cout << ex.what() << endl;
    }
}

IplImage* WinCameraService::getImage(){
	if(!myVideo.isFrameNew(myDevice)){
		return NULL;
	}
	myVideo.getPixels(myDevice,myImageData, true);
	int w = myVideo.getWidth(myDevice);
	int h = myVideo.getHeight(myDevice);
	CvSize size = cvSize(w,h);
	IplImage* img = cvCreateImageHeader(size, IPL_DEPTH_8U, 3);
	if(img == NULL){
		return NULL;
	}
	cvInitImageHeader(img, size, IPL_DEPTH_8U, 3, IPL_ORIGIN_BL);
    img->imageData = (char*)myImageData;
    img->imageDataOrigin = img->imageData;

	//my camera image is inverted so we flip it
	cvFlip(img, img);
	return img;
}

void WinCameraService::handleCommandMessage(Message &msg){}


int WinCameraService::getFrameLength(){
    return myFrameLength;
}

void WinCameraService::setFrameLength(int len){
    myFrameLength = len;
}